# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/15 11:16
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: langchain_instance.py
from typing import List, Optional, Dict, Any

from langchain_core.messages import BaseMessage, get_buffer_string
from langchain.memory import ConversationSummaryBufferMemory, ConversationTokenBufferMemory
from langchain_core.output_parsers import StrOutputParser

from agentuniverse.agent.memory.enum import ChatMessageEnum
from agentuniverse.agent.memory.message import Message
from agentuniverse.prompt.prompt import Prompt
from agentuniverse.prompt.prompt_manager import PromptManager


class AuConversationSummaryBufferMemory(ConversationSummaryBufferMemory):
    """Long term memory to store conversation memory.

    If memories exceeds the max tokens limit,
    memories will be compressed by the AuConversationSummaryBufferMemory.predict_new_summary method.

    Attributes:
        messages (List[BaseMessage]): List of BaseMessage to save.
    """

    messages: Optional[List[Message]] = None
    prompt_version: Optional[str] = None

    def __init__(self, **kwargs):
        """The __init__ method.

        The initialization method saves the memory context from messages in `messages` attribute to the memory buffer.

        Args:
            **kwargs: Arbitrary keyword arguments.
        """

        super().__init__(**kwargs)
        self.build_memory()

    @property
    def load_memory(self) -> List[BaseMessage]:
        """ General method: load the memory context from the memory buffer."""
        messages = self.chat_memory.messages
        if self.moving_summary_buffer != "":
            moving_messages: List[BaseMessage] = [
                self.summary_message_cls(content=self.moving_summary_buffer)
            ]
            messages = moving_messages + messages
        return messages

    @property
    def load_memory_str(self) -> str:
        """ General method: load the memory context from the memory buffer as string format."""
        buffer = get_buffer_string(self.chat_memory.messages)
        if self.moving_summary_buffer is not None:
            buffer = self.moving_summary_buffer + buffer
        return buffer

    def build_memory(self):
        """Save the memory context from messages in `messages` attribute to the memory buffer.

        Note:
            If conversation messages contains the system message, it will be saved in the `moving_summary_buffer` attribute.
        """
        if self.messages is None:
            return
        # save system messages
        for message in self.messages:
            if message.type.lower() == ChatMessageEnum.SYSTEM.value:
                self.moving_summary_buffer += message.content
                self.messages.remove(message)
        # generate pairs of Human and AI conversation messages and save to the memory buffer
        for i in range(0, len(self.messages), 2):
            inputs, outputs = self.generate_chat_messages(self.messages[i], self.messages[i + 1])
            self.save_context(inputs, outputs)

    def generate_chat_messages(self, *pairs: Message):
        """generate pairs of Human and AI conversation messages"""
        human_dict = {}
        ai_dict = {}
        for pair in pairs:
            if pair.type.lower() == ChatMessageEnum.HUMAN.value:
                human_dict = {self.input_key: pair.content}
            elif pair.type.lower() == ChatMessageEnum.AI.value:
                ai_dict = {self.output_key: pair.content}

        return human_dict, ai_dict

    def save_context(self, inputs: Dict[str, Any], outputs: Dict[str, str]) -> None:
        """Save context from the conversation to buffer and prune memories"""

        def message_to_dict(message):
            return {
                "content": message.content,
                "type": message.type
            }

        super().save_context(inputs, outputs)
        message_list = self.load_memory
        messages_dicts = [message_to_dict(message) for message in message_list]
        inputs[self.memory_key] = messages_dicts

    def predict_new_summary(
            self, messages: List[BaseMessage], existing_summary: str
    ) -> str:
        """Predict new summary, summarize memories in multiple rounds of conversations."""
        new_lines = get_buffer_string(
            messages,
            human_prefix=self.human_prefix,
            ai_prefix=self.ai_prefix,
        )
        prompt_version = self.prompt_version if self.prompt_version else 'chat_memory.summarizer_cn'
        prompt: Prompt = PromptManager().get_instance_obj(prompt_version)
        chain = prompt.as_langchain() | self.llm | StrOutputParser()
        return chain.invoke(input={'summary': existing_summary, 'new_lines': new_lines, 'max_tokens': 500})


class AuConversationTokenBufferMemory(ConversationTokenBufferMemory):
    """Short term memory to store conversation memory.

    If memories exceeds the max tokens limit, the requirement is met by truncating part of memories

    Attributes:
        messages (List[BaseMessage]): List of BaseMessage to save.
    """

    messages: Optional[List[Message]] = None

    def __init__(self, **kwargs):
        """The __init__ method.

        The initialization method saves the memory context from messages in `messages` attribute to the memory buffer.

        Args:
            **kwargs: Arbitrary keyword arguments.
        """
        super().__init__(**kwargs)
        self.build_memory()

    @property
    def load_memory(self) -> List[BaseMessage]:
        """ General method: load the memory context from the memory buffer."""
        return self.chat_memory.messages

    @property
    def load_memory_str(self) -> str:
        """ General method: load the memory context from the memory buffer as string format."""
        return get_buffer_string(self.chat_memory.messages)

    def build_memory(self):
        """Save the memory context from messages in `messages` attribute to the memory buffer.

        Note:
            If conversation messages contains the system message, it will be discarded from the memory.
        """
        if self.messages is None:
            return
        # discard system messages
        for message in self.messages:
            if message.type.lower() == ChatMessageEnum.SYSTEM.value:
                self.messages.remove(message)
        # generate pairs of Human and AI conversation messages and save to the memory buffer
        for i in range(0, len(self.messages), 2):
            inputs, outputs = self.generate_chat_messages(self.messages[i], self.messages[i + 1])
            self.save_context(inputs, outputs)

    def generate_chat_messages(self, *pairs: Message):
        """generate pairs of Human and AI conversation messages"""
        human_dict = {}
        ai_dict = {}
        for pair in pairs:
            if pair.type == ChatMessageEnum.HUMAN.value:
                human_dict = {self.input_key: pair.content}
            elif pair.type == ChatMessageEnum.AI.value:
                ai_dict = {self.output_key: pair.content}

        return human_dict, ai_dict

    def save_context(self, inputs: Dict[str, Any], outputs: Dict[str, str]) -> None:
        """Save context from the conversation to buffer and truncate part of memories."""

        def message_to_dict(message):
            return {
                "content": message.content,
                "type": message.type
            }

        super().save_context(inputs, outputs)
        message_list = self.load_memory
        messages_dicts = [message_to_dict(message) for message in message_list]
        inputs[self.memory_key] = messages_dicts



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/12 23:16
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: component_configer.py

from typing import Optional
from agentuniverse.base.config.configer import Configer


class ComponentConfiger(object):
    """The ComponentConfiger class, which is used to load and manage the component configuration."""

    def __init__(self, configer: Optional[Configer] = None):
        """Initialize the ComponentConfiger."""
        self.__configer: Optional[Configer] = configer
        self.__metadata_type: Optional[str] = None
        self.__metadata_module: Optional[str] = None
        self.__metadata_class: Optional[str] = None

    @property
    def configer(self) -> Optional[Configer]:
        """Return the Configer object."""
        return self.__configer

    @property
    def metadata_type(self) -> Optional[str]:
        """Return the type of the component."""
        return self.__metadata_type

    @property
    def metadata_module(self) -> Optional[str]:
        """Return the module of the component."""
        return self.__metadata_module

    @metadata_module.setter
    def metadata_module(self, metadata_module: str):
        self.__metadata_module = metadata_module

    @property
    def metadata_class(self) -> Optional[str]:
        """Return the class of the component."""
        return self.__metadata_class

    @metadata_class.setter
    def metadata_class(self, metadata_class: str):
        self.__metadata_class = metadata_class

    def load(self) -> 'ComponentConfiger':
        """Load the configuration by the Configer object.
        Returns:
            ComponentConfiger: the ComponentConfiger object
        """
        return self.load_by_configer(self.configer)

    def load_by_configer(self, configer: Configer) -> 'ComponentConfiger':
        """Load the configuration by the Configer object.
        Args:
            configer(Configer): the Configer object
        Returns:
            ComponentConfiger: the ComponentConfiger object
        """
        self.__configer = configer

        try:
            for k, v in configer.value.items():
                self.__dict__[k] = v
            self.__metadata_type = configer.value.get('metadata').get('type')
            self.__metadata_module = configer.value.get('metadata').get('module')
            self.__metadata_class = configer.value.get('metadata').get('class')
        except Exception as e:
            raise Exception(f"Failed to parse the component configuration: {e}")

        return self

    def get_component_config_type(self) -> Optional[str]:
        """Return the type of the component.

        Returns:
            Optional[str]: the type of the component
        """
        return self.__metadata_type



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/12 20:02
# @Author  : heji
# @Email   : lc299034@antgroup.com
# @FileName: agent_manager.py

"""Agents manager."""
from agentuniverse.base.annotation.singleton import singleton
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.component.component_manager_base import ComponentManagerBase


@singleton
class AgentManager(ComponentManagerBase):
    """The AgentManager class, which is used to manage the agents."""

    def __init__(self):
        """Initialize the Agent manager."""
        super().__init__(ComponentEnum.AGENT)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/6/12 11:43
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: dashscope_embedding.py
import aiohttp
import requests
from typing import List, Generator, Optional
from pydantic import Field
import json

from agentuniverse.base.util.env_util import get_from_env
from agentuniverse.agent.action.knowledge.embedding.embedding import Embedding

# Dashscope support max 25 string in one batch, each string max tokens is 2048.
DASHSCOPE_MAX_BATCH_SIZE = 25
DASHSCOPE_EMBEDDING_URL = "https://dashscope.aliyuncs.com/api/v1/services/embeddings/text-embedding/text-embedding"


def batched(inputs: List,
            batch_size: int = DASHSCOPE_MAX_BATCH_SIZE) -> Generator[List, None, None]:
    # Split input string list, due to dashscope support 25 strings in one call.
    for i in range(0, len(inputs), batch_size):
        yield inputs[i:i + batch_size]


class DashscopeEmbedding(Embedding):
    """The Dashscope embedding class."""
    dashscope_api_key: Optional[str] = Field(
        default_factory=lambda: get_from_env("DASHSCOPE_API_KEY")
    )

    def get_embeddings(self, texts: List[str], **kwargs) -> List[List[float]]:
        """
        Retrieve text embeddings for a list of input texts.

        This function interfaces with the DashScope embedding API to obtain
        embeddings for a batch of input texts. It handles batching of input texts
        to ensure efficient API calls. Each text is processed using the specified
        embedding model.

        Args:
            texts (List[str]): A list of input texts to be embedded.

        Returns:
            List[List[float]]: A list of embeddings corresponding to the input texts.

        Raises:
            Exception: If the API call to DashScope fails, an exception is raised with
                       the respective error code and message.
        """
        def post(post_params):
            response = requests.post(
                url=DASHSCOPE_EMBEDDING_URL,
                headers={
                    "Content-Type": "application/json",
                    "Authorization": f"Bearer {self.dashscope_api_key}"
                },
                data=json.dumps(post_params, ensure_ascii=False).encode(
                    "utf-8"),
                timeout=300
            )
            resp_json = response.json()
            return resp_json
        if not self.dashscope_api_key:
            raise Exception("No DASHSCOPE_API_KEY in your environment.")
        result = []
        post_params = {
            "model": self.embedding_model_name,
            "input": {},
            "parameters": {
            }
        }
        if self.embedding_model_name == "text-embedding-v3":
            post_params["parameters"][
                "dimension"] = self.embedding_dims if self.embedding_dims else 1024
            post_params["parameters"][
                "output_type"] = kwargs["output_type"] if "output_type" in kwargs else "dense"
        post_params["parameters"]["text_type"] = kwargs["text_type"] if "text_type" in kwargs else "document"

        for batch in batched(texts):
            post_params["input"]["texts"] = batch
            resp_json: dict = post(post_params)
            data = resp_json.get("output")
            if data:
                data = data["embeddings"]
                batch_result = [d['embedding'] for d in data if 'embedding' in d]
                result += batch_result
            else:
                error_code = resp_json.get("code", "")
                error_message = resp_json.get("message", "")
                raise Exception(f"Failed to call dashscope embedding api, "
                                f"error code:{error_code}, "
                                f"error message:{error_message}")
        return result

    async def async_get_embeddings(self, texts: List[str], **kwargs) -> List[List[float]]:
        """
        Async version of get_embeddings.

        This function interfaces with the DashScope embedding API to obtain
        embeddings for a batch of input texts. It handles batching of input texts
        to ensure efficient API calls. Each text is processed using the specified
        embedding model.

        Args:
            texts (List[str]): A list of input texts to be embedded.

        Returns:
            List[List[float]]: A list of embeddings corresponding to the input texts.

        Raises:
            Exception: If the API call to DashScope fails, an exception is raised with
                       the respective error code and message.
        """
        async def async_post(post_params):
            async with aiohttp.ClientSession() as session:
                async with await session.post(
                        url=DASHSCOPE_EMBEDDING_URL,
                        headers={
                            "Content-Type": "application/json",
                            "Authorization": f"Bearer {self.dashscope_api_key}"
                        },
                        data=json.dumps(post_params, ensure_ascii=False).encode(
                            "utf-8"),
                        timeout=300,
                ) as resp:
                    resp_json = await resp.json()
            return resp_json
        if not self.dashscope_api_key:
            raise Exception("No DASHSCOPE_API_KEY in your environment.")
        result = []
        post_params = {
            "model": self.embedding_model_name,
            "input": {},
            "parameters": {
                "text_type": "query"
            }
        }
        if self.embedding_model_name == "text-embedding-v3":
            post_params["parameters"][
                "dimension"] = self.embedding_dims if self.embedding_dims else 1024
            post_params["parameters"][
                "output_type"] = kwargs[
                "output_type"] if "output_type" in kwargs else "dense"
        post_params["parameters"]["text_type"] = kwargs[
            "text_type"] if "text_type" in kwargs else "document"

        for batch in batched(texts):
            post_params["input"]["texts"] = batch
            resp_json: dict = await async_post(post_params)
            data = resp_json.get("output")
            if data:
                data = data["embeddings"]
                batch_result = [d['embedding'] for d in data if
                                'embedding' in d]
                result += batch_result
            else:
                error_code = resp_json.get("code", "")
                error_message = resp_json.get("message", "")
                raise Exception(f"Failed to call dashscope embedding api, "
                                f"error code:{error_code}, "
                                f"error message:{error_message}")
        return result


# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/10/17 20:36
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: planning_agent_template.py
from queue import Queue

from langchain_core.utils.json import parse_json_markdown

from agentuniverse.agent.input_object import InputObject
from agentuniverse.agent.template.agent_template import AgentTemplate
from agentuniverse.base.config.component_configer.configers.agent_configer import AgentConfiger
from agentuniverse.base.util.common_util import stream_output
from agentuniverse.base.util.logging.logging_util import LOGGER


class PlanningAgentTemplate(AgentTemplate):

    def input_keys(self) -> list[str]:
        return ['input']

    def output_keys(self) -> list[str]:
        return ['framework', 'thought']

    def parse_input(self, input_object: InputObject, agent_input: dict) -> dict:
        agent_input['input'] = input_object.get_data('input')
        agent_input['expert_framework'] = input_object.get_data('expert_framework', {}).get('planning')
        return agent_input

    def parse_result(self, agent_result: dict) -> dict:
        final_result = dict()

        output = agent_result.get('output')
        output = parse_json_markdown(output)
        final_result['framework'] = output.get('framework')
        final_result['thought'] = output.get('thought', '')

        # add planning agent log info.
        logger_info = f"\nPlanning agent execution result is :\n"
        for index, one_framework in enumerate(final_result.get('framework')):
            logger_info += f"[{index + 1}] {one_framework} \n"
        LOGGER.info(logger_info)
        return final_result

    def initialize_by_component_configer(self, component_configer: AgentConfiger) -> 'PlanningAgentTemplate':
        super().initialize_by_component_configer(component_configer)
        self.prompt_version = self.agent_model.profile.get('prompt_version', 'default_planning_agent.cn')
        self.validate_required_params()
        return self

    def validate_required_params(self):
        if not self.llm_name:
            raise ValueError(f'llm_name of the agent {self.agent_model.info.get("name")}'
                             f' is not set, please go to the agent profile configuration'
                             ' and set the `name` attribute in the `llm_model`.')

    def add_output_stream(self, output_stream: Queue, agent_output: str) -> None:
        if not output_stream:
            return
        # add planning agent final result into the stream output.
        stream_output(output_stream,
                      {"data": {
                          'output': parse_json_markdown(agent_output).get('framework'),
                          "agent_info": self.agent_model.info
                      }, "type": "planning"})



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/7/23 14:12
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: doc_processor_manager.py

from agentuniverse.base.annotation.singleton import singleton
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.component.component_manager_base import ComponentManagerBase
from agentuniverse.agent.action.knowledge.doc_processor.doc_processor import DocProcessor


@singleton
class DocProcessorManager(ComponentManagerBase[DocProcessor]):
    """A singleton manager class of the DocProcessor."""

    def __init__(self):
        super().__init__(ComponentEnum.DOC_PROCESSOR)


# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/5/30 10:22
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: milvus_store.py
from typing import List, Optional, Any

try:
    from pymilvus import connections, Collection, CollectionSchema, \
        FieldSchema, DataType, utility
except ImportError as e:
    raise ImportError(
        "pymilvus is not installed. Please install it with 'pip install pymilvus'") from e

from agentuniverse.agent.action.knowledge.store.document import Document
from agentuniverse.agent.action.knowledge.store.query import Query
from agentuniverse.agent.action.knowledge.store.store import Store
from agentuniverse.agent.action.knowledge.embedding.embedding_manager import EmbeddingManager
from agentuniverse.base.config.component_configer.component_configer import \
    ComponentConfiger

DEFAULT_CONNECTION_ARGS = {
    "host": "localhost",
    "port": "19530"
}

DEFAULT_SEARCH_ARGS = {
    "metric_type": "L2",
    "params": {"nprobe": 10}
}

DEFAULT_INDEX_PARAMS = {
    "metric_type": "L2",
    "index_type": "HNSW",
    "params": {"M": 8, "efConstruction": 64},
}


class MilvusStore(Store):
    connection_args: Optional[dict] = None
    search_args: Optional[dict] = None
    index_params: Optional[dict] = None
    collection_name: Optional[str] = 'milvus_db'
    collection: Collection = None
    connection_name: str = 'default_connection'
    embedding_model: Optional[str] = None
    similarity_top_k: Optional[int] = 10
    query_embedding: bool = False


    def _connect_to_milvus(self, connection_args: dict):
        """Connect to Milvus server."""
        if not connections.has_connection(self.connection_name):
            connections.connect(
                alias=self.connection_name, **connection_args
            )

    def _new_client(self) -> Any:
        host = self.connection_args["host"]
        port = self.connection_args["port"]
        db_name = self.connection_args.get("db_name", "default")
        self.connection_name = f"{host}_{port}_{db_name}"
        self._connect_to_milvus(self.connection_args)
        if utility.has_collection(self.collection_name,
                                  using=self.connection_name):
            self.collection = Collection(
                self.collection_name, using=self.connection_name
            )
            self.collection.load()

    def _initialize_by_component_configer(self,
                                          milvus_store_configer: ComponentConfiger) -> 'DocProcessor':
        super()._initialize_by_component_configer(milvus_store_configer)
        if hasattr(milvus_store_configer, "connection_args"):
            self.connection_args = milvus_store_configer.connection_args
        else:
            self.connection_args = DEFAULT_CONNECTION_ARGS
        if hasattr(milvus_store_configer, "search_args"):
            self.search_args = milvus_store_configer.search_args
        else:
            self.search_args = DEFAULT_SEARCH_ARGS
        if hasattr(milvus_store_configer, "index_params"):
            self.index_params = milvus_store_configer.index_params
        else:
            self.index_params = DEFAULT_INDEX_PARAMS
        if hasattr(milvus_store_configer, "collection_name"):
            self.collection_name = milvus_store_configer.collection_name
        if hasattr(milvus_store_configer, "embedding_model"):
            self.embedding_model = milvus_store_configer.embedding_model
        if hasattr(milvus_store_configer, "similarity_top_k"):
            self.similarity_top_k = milvus_store_configer.similarity_top_k
        if hasattr(milvus_store_configer, "query_embedding"):
            self.similarity_top_k = milvus_store_configer.query_embedding
        return self

    def _create_or_load_collection(self,
                                   dim: int = 128,
                                   max_length: int = 65535,
                                   index_params: dict = None):
        """
        Create a new collection or load an existing collection.

        This method handles the creation of a new collection with specified parameters
        or loads an existing collection if it already exists. Collections are used
        to store data with specific dimensional attributes and indexing parameters.

        Parameters:
        - dim (int): The dimension of the collection, default is 128.
        - max_length (int): The maximum length of the collection, default is 65535.
        - index_params (dict, optional): Additional parameters for indexing. This
          dictionary can include specific configurations for the index creation or loading.

        Returns:
        None
        """
        if utility.has_collection(self.collection_name,
                                  using=self.connection_name):
            self.collection = Collection(
                self.collection_name, using=self.connection_name
            )
        else:
            if not index_params:
                index_params = self.index_params
            fields = [
                FieldSchema(name="id", dtype=DataType.VARCHAR, max_length=100,
                            is_primary=True),
                FieldSchema(name="embedding", dtype=DataType.FLOAT_VECTOR,
                            dim=dim),
                FieldSchema(name="text", dtype=DataType.VARCHAR,
                            max_length=max_length),
                FieldSchema(name="metadata", dtype=DataType.JSON)
            ]
            schema = CollectionSchema(fields, "Milvus collection schema")
            self.collection = Collection(self.collection_name, schema,
                                         using=self.connection_name)
            self.collection.create_index(
                field_name="embedding",
                index_params=index_params
            )
            self.collection.load()

    def query(self,
              query: Query,
              search_args: dict = None,
              **kwargs) -> List[Document]:
        """
        Query the Milvus collection with the given query and return the top k results.

        Parameters:
        - query (Query): The query object that contains the parameters and data for the search.
        - search_args (dict, optional): A dictionary of additional arguments for the search.
          This can include parameters such as the number of results to return, specific search
          algorithms, or other configurations.

        Returns:
        - List[Document]: A list of Document objects that are the top k results from the query.
        """
        if not self.collection:
            return self.to_documents([])
        embedding = query.embeddings
        if self.embedding_model is not None and len(embedding) == 0:
            if not self.embedding_model:
                raise Exception("Milvus store can only save vector, "
                                "you should provide embedding in your document or specify an embedding model.")
            embedding = EmbeddingManager().get_instance_obj(
                    self.embedding_model
                ).get_embeddings([query.query_str], text_type="query")

        if not search_args:
            search_args = self.search_args
        if len(embedding) > 0:
            if self.query_embedding:
                output_fields = ["id", "text", "embedding", "metadata"]
            else:
                output_fields = ["id", "text", "metadata"]
            query_result = self.collection.search(
                data=embedding,
                anns_field="embedding",
                param=search_args,
                limit=query.similarity_top_k if query.similarity_top_k else self.similarity_top_k,
                output_fields=output_fields
            )
        else:
            query_result = []
        return self.to_documents(query_result)


    def upsert_document(self,
                        documents: List[Document],
                        max_length: int = 65535,
                        index_params: dict = None,
                        **kwargs):
        """
        This method inserts new documents or updates existing documents in the collection.
        The upsert operation ensures that the documents are either added if they do not
        already exist or updated if they do.

        Parameters:
        - documents (List[Document]): A list of Document objects to be upserted into the collection.
        - max_length (int): The maximum length of the collection, default is 65535.
        - index_params (dict, optional): Additional parameters for indexing. This dictionary
          can include specific configurations for the index creation or updating.

        Returns:
        None
        """
        for document in documents:
            embedding = document.embedding
            if len(embedding) == 0:
                if not self.embedding_model:
                    raise Exception("Milvus store can only save vector, "
                                    "you should provide embedding in your document or specify an embedding model.")
                else:
                    embedding = EmbeddingManager().get_instance_obj(
                        self.embedding_model
                    ).get_embeddings([document.text])[0]

            if not self.collection:
                self._create_or_load_collection(
                    dim=len(embedding),
                    max_length=max_length,
                    index_params=index_params
                )
            expr = f'id == "{document.id}"'
            existing_docs = self.collection.query(expr)
            entities = [
                [document.id],
                [embedding],
                [document.text],
                [document.metadata]
            ]
            if existing_docs:
                self.collection.delete(expr)
                self.collection.insert(entities)
            else:
                self.collection.insert(entities)
            self.collection.load()

    def insert_document(self,
                         documents: List[Document],
                         max_length: int = 65535,
                         index_params: dict = None,
                         ):
        """Insert documents to the Milvus collection."""
        self.upsert_document(documents, max_length, index_params)

    def update_document(self, documents: List[Document], **kwargs):
        """Update document into the store."""
        self.upsert_document(documents)

    @staticmethod
    def to_documents(query_result) -> List[Document]:
        """Convert the query results of Milvus to the AgentUniverse(AU)
        document format."""
        if query_result is None:
            return []
        documents = []
        for result in query_result:
            for res in result:
                documents.append(Document(
                    id=res.fields.get("id"),
                    text=res.fields.get("text"),
                    embedding=res.fields.get("embedding")
                    if res.fields.get("embedding") else [],
                    metadata=res.fields.get("metadata")
                    if res.fields.get("metadata") else None)
                )
        return documents



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/10/24 21:19
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: rag_template.py
from agentuniverse.agent.input_object import InputObject
from agentuniverse.agent.memory.memory import Memory
from agentuniverse.agent.template.agent_template import AgentTemplate
from agentuniverse.base.config.component_configer.configers.agent_configer import AgentConfiger
from agentuniverse.llm.llm import LLM
from agentuniverse.prompt.prompt import Prompt


class RagAgentTemplate(AgentTemplate):

    def input_keys(self) -> list[str]:
        return ['input']

    def output_keys(self) -> list[str]:
        return ['output']

    def parse_input(self, input_object: InputObject, agent_input: dict) -> dict:
        agent_input['input'] = input_object.get_data('input')
        return agent_input

    def parse_result(self, agent_result: dict) -> dict:
        return {**agent_result, 'output': agent_result['output']}

    def customized_execute(self, input_object: InputObject, agent_input: dict, memory: Memory, llm: LLM, prompt: Prompt,
                           **kwargs) -> dict:
        tool_res: str = self.invoke_tools(input_object)
        knowledge_res: str = self.invoke_knowledge(agent_input.get('input'), input_object)
        agent_input['background'] = (agent_input['background']
                                     + f"tool_res: {tool_res} \n\n knowledge_res: {knowledge_res}")
        return super().customized_execute(input_object, agent_input, memory, llm, prompt, **kwargs)

    async def customized_async_execute(self, input_object: InputObject, agent_input: dict, memory: Memory, llm: LLM,
                                       prompt: Prompt, **kwargs) -> dict:
        tool_res: str = self.invoke_tools(input_object)
        knowledge_res: str = self.invoke_knowledge(agent_input.get('input'), input_object)
        agent_input['background'] = (agent_input['background']
                                     + f"tool_res: {tool_res} \n\n knowledge_res: {knowledge_res}")
        return await super().customized_async_execute(input_object, agent_input, memory, llm, prompt, **kwargs)

    def initialize_by_component_configer(self, component_configer: AgentConfiger) -> 'RagAgentTemplate':
        super().initialize_by_component_configer(component_configer)
        self.prompt_version = self.agent_model.profile.get('prompt_version', 'default_rag_agent.cn')
        return self



# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Time    : 2024/3/29 15:31
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: framework_context.py

from typing import Dict, Any

from agentuniverse.base.context.framework_context_manager import FrameworkContextManager


class FrameworkContext:
    """A context class that provides a thread level variable dictionary."""

    def __init__(self, context: Dict[str, Any]):
        """Save context dict and init an empty dict to save prior context.

        Args:
            context (`dict`):
                A dict contains all kv pairs
                which will be added to current context.
        """
        self.context = context
        self.old_state = {}

    def __enter__(self):
        """Preserve the prior context and set a new one."""
        for key, value in self.context.items():
            if FrameworkContextManager().is_context_exist(key):
                self.old_state[key] = FrameworkContextManager().get_context(key)
            FrameworkContextManager().set_context(key, value)

    def __exit__(self, exc_type, exc_value, traceback):
        """Clear the current context and revert to the prior context."""
        for key, value in self.context.items():
            if key in self.old_state:
                FrameworkContextManager().set_context(key, self.old_state[key])
            else:
                FrameworkContextManager().del_context(key)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/12 17:47
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: component_enum.py
from enum import Enum


class ComponentEnum(Enum):
    """The enumeration of the supported components."""
    AGENT = "AGENT"
    KNOWLEDGE = "KNOWLEDGE"
    LLM = "LLM"
    PLANNER = "PLANNER"
    TOOL = "TOOL"
    DEFAULT = "DEFAULT"
    SERVICE = "SERVICE"
    MEMORY = "MEMORY"
    PROMPT = "PROMPT"
    SQLDB_WRAPPER = "SQLDB_WRAPPER"
    PRODUCT = "PRODUCT"
    WORKFLOW = "WORKFLOW"
    EMBEDDING = "EMBEDDING"
    DOC_PROCESSOR = "DOC_PROCESSOR"
    READER = "READER"
    STORE = "STORE"
    RAG_ROUTER = "RAG_ROUTER"
    QUERY_PARAPHRASER = "QUERY_PARAPHRASER"
    WORK_PATTERN = "WORK_PATTERN"
    MEMORY_COMPRESSOR = "MEMORY_COMPRESSOR"
    MEMORY_STORAGE = "MEMORY_STORAGE"

    @staticmethod
    def to_value_list():
        """Return the value list of the enumeration."""
        return [item.value for item in ComponentEnum]

    @staticmethod
    def from_value(value):
        """Return the enum member corresponding to the given value."""
        for item in ComponentEnum:
            if item.value == value:
                return item
        raise ValueError(f"No enum member with value {value}")



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/13 12:01
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: knowledge_configer.py
from typing import Optional, Dict, List
from agentuniverse.base.config.component_configer.component_configer import ComponentConfiger
from agentuniverse.base.config.configer import Configer


class KnowledgeConfiger(ComponentConfiger):
    """The KnowledgeConfiger class, which is used to load and manage the Knowledge configuration."""

    def __init__(self, configer: Optional[Configer] = None):
        """Initialize the KnowledgeConfiger."""
        super().__init__(configer)
        self.__name: Optional[str] = None
        self.__description: Optional[str] = None
        self.__ext_info: Optional[Dict] = None
        self.stores: List[str] = []
        self.query_paraphrasers: Optional[List[str]] = []
        self.insert_processors: Optional[List[str]] = []
        self.rag_router: str = "base_router"
        self.post_processors: List[str] = []
        self.readers: Dict[str, str] = dict()

    @property
    def name(self) -> Optional[str]:
        """Return the name of the Knowledge."""
        return self.__name

    @property
    def description(self) -> Optional[str]:
        """Return the description of the Knowledge."""
        return self.__description

    @property
    def ext_info(self) -> Optional[Dict]:
        return self.__ext_info

    def load(self) -> 'KnowledgeConfiger':
        """Load the configuration by the Configer object.
        Returns:
            KnowledgeConfiger: the KnowledgeConfiger object
        """
        return self.load_by_configer(self.__configer)

    def load_by_configer(self, configer: Configer) -> 'KnowledgeConfiger':
        """Load the configuration by the Configer object.
        Args:
            configer(Configer): the Configer object
        Returns:
            KnowledgeConfiger: the KnowledgeConfiger object
        """
        super().load_by_configer(configer)

        try:
            self.__name = configer.value.get('name')
            self.__description = configer.value.get('description')
            self.__ext_info = configer.value.get('ext_info')
        except Exception as e:
            raise Exception(f"Failed to parse the Knowledge configuration: {e}")
        return self



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/7/24 16:29
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: product.py
from typing import Optional
import os

from agentuniverse.base.component.component_base import ComponentBase
from agentuniverse.base.component.component_configer_util import ComponentConfigerUtil
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.config.application_configer.application_config_manager import ApplicationConfigManager
from agentuniverse_product.base.product_configer import ProductConfiger


class Product(ComponentBase):
    """The basic class of the product."""

    component_type: ComponentEnum = ComponentEnum.PRODUCT
    id: Optional[str] = None
    nickname: Optional[str] = None
    type: Optional[str] = None
    avatar: Optional[str] = None
    description: Optional[str] = None
    _instance: Optional[ComponentBase] = None

    @property
    def instance(self) -> ComponentBase:
        return self._instance

    def get_ctime(self) -> float:
        """Return the product creation time."""
        return os.path.getctime(self.component_config_path)

    def get_mtime(self) -> float:
        """Return the product last modification time."""
        return os.path.getmtime(self.component_config_path)

    def get_instance_code(self) -> str:
        """Return the full name of the product."""
        appname = ApplicationConfigManager().app_configer.base_info_appname
        return f"{appname}.product.{self.id}"

    def initialize_by_component_configer(self,
                                         product_configer: ProductConfiger) -> 'Product':
        """Initialize the Product by the ProductConfiger object.

        Args:
            product_configer(ProductConfiger): A configer contains product basic info.
        Returns:
            Product: A Product instance.
        """
        if product_configer.nickname:
            self.nickname = product_configer.nickname
        if product_configer.id:
            self.id = product_configer.id
        if product_configer.type:
            self.type = product_configer.type
        if product_configer.avatar:
            self.avatar = product_configer.avatar
        if product_configer.description:
            self.description = product_configer.description
        self.init_instance()
        return self

    def init_instance(self):
        """Initialize the specific component instance corresponding to the product."""
        component_manager_clz = ComponentConfigerUtil.get_component_manager_clz_by_type(
            ComponentEnum.from_value(self.type))
        self._instance = component_manager_clz().get_instance_obj(self.id)
        if self._instance is None:
            raise ValueError(f"The aU instance corresponding to the product id does not exist,"
                             f" please check the product type and id parameters.")



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/5/21 13:52
# @Author  : weizjajj
# @Email   : weizhongjie.wzj@antgroup.com
# @FileName: baichuan_openai_style_llm.py
from typing import Optional, Any, Union, Iterator, AsyncIterator

from pydantic import Field

from agentuniverse.base.annotation.trace import trace_llm
from agentuniverse.base.util.env_util import get_from_env
from agentuniverse.llm.llm_output import LLMOutput
from agentuniverse.llm.openai_style_llm import OpenAIStyleLLM

BAICHUAN_Max_CONTEXT_LENGTH = {
    "Baichuan2-Turbo": 8000,
    "Baichuan2-Turbo-192k": 192000,
    "Baichuan3-Turbo": 8000,
    "Baichuan3-Turbo-128k": 128000,
    "Baichuan4": 8000
}


class BAICHUANOpenAIStyleLLM(OpenAIStyleLLM):
    """
    BAICHUAN OpenAI Style LLM
    Args:
        api_key (Optional[str]): API key for the model. Defaults to None.
        api_base (Optional[str]): API base URL for the model. Defaults to "https://api.openai.com/v1".
    """

    api_key: Optional[str] = Field(default_factory=lambda: get_from_env("BAICHUAN_API_KEY"))
    api_base: Optional[str] = "https://api.baichuan-ai.com/v1"
    proxy: Optional[str] = Field(default_factory=lambda: get_from_env("BAICHUAN_PROXY"))
    organization: Optional[str] = Field(default_factory=lambda: get_from_env("BAICHUAN_ORGANIZATION"))

    def max_context_length(self) -> int:
        if super().max_context_length():
            return super().max_context_length()
        return BAICHUAN_Max_CONTEXT_LENGTH.get(self.model_name, 8000)

    def _call(self, messages: list, **kwargs: Any) -> Union[LLMOutput, Iterator[LLMOutput]]:
        """ The call method of the LLM.

        Users can customize how the model interacts by overriding call method of the LLM class.

        Args:
            messages (list): The messages to send to the LLM.
            **kwargs: Arbitrary keyword arguments.
        """
        return super()._call(messages, **kwargs)

    async def _acall(self, messages: list, **kwargs: Any) -> Union[LLMOutput, AsyncIterator[LLMOutput]]:
        """ The async call method of the LLM.

        Users can customize how the model interacts by overriding acall method of the LLM class.

        Args:
            messages (list): The messages to send to the LLM.
            **kwargs: Arbitrary keyword arguments.
        """
        return await super()._acall(messages, **kwargs)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/13 12:01
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: tool_configer.py
from typing import Optional, List
from agentuniverse.base.config.component_configer.component_configer import ComponentConfiger
from agentuniverse.base.config.configer import Configer


class ToolConfiger(ComponentConfiger):
    """The ToolConfiger class, which is used to load and manage the Tool configuration."""

    def __init__(self, configer: Optional[Configer] = None):
        """Initialize the ToolConfiger."""
        super().__init__(configer)
        self.__name: Optional[str] = None
        self.__description: Optional[str] = None
        self.__tool_type: Optional[str] = None
        self.__input_keys: Optional[List] = None

    @property
    def name(self) -> Optional[str]:
        """Return the name of the Tool."""
        return self.__name

    @property
    def description(self) -> Optional[str]:
        """Return the description of the Tool."""
        return self.__description

    @property
    def tool_type(self) -> Optional[str]:
        return self.__tool_type

    @property
    def input_keys(self) -> Optional[List]:
        return self.__input_keys

    def load(self) -> 'ToolConfiger':
        """Load the configuration by the Configer object.
        Returns:
            ToolConfiger: the ToolConfiger object
        """
        return self.load_by_configer(self.__configer)

    def load_by_configer(self, configer: Configer) -> 'ToolConfiger':
        """Load the configuration by the Configer object.
        Args:
            configer(Configer): the Configer object
        Returns:
            ToolConfiger: the ToolConfiger object
        """
        super().load_by_configer(configer)

        try:
            self.__name = configer.value.get('name')
            self.__description = configer.value.get('description')
            self.__tool_type = configer.value.get('tool_type')
            self.__input_keys = configer.value.get('input_keys')
        except Exception as e:
            raise Exception(f"Failed to parse the Tool configuration: {e}")
        return self



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/7/29 12:14
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: yaml_util.py
import os

from ruamel.yaml import YAML
from typing import Any, Dict
import os


def update_nested_yaml_value(config_path: str, updates: dict) -> None:
    """Update nested values in YAML

    Args:
        config_path(str): The path to the YAML file.

        updates(dict): A dictionary of key-value pairs to update in the YAML file.
        The keys should be dot-separated paths to the target values,
        and the values should be the new values to set.

    Examples:
        update_nested_yaml_value(
        config_path='/xxx/agentUniverse/sample_standard_app/intelligence/agentic/agent/agent_instance/rag_agent_case/demo_rag_agent.yaml',
        updates={'info.description': 'demo rag agent',
          'profile.llm_model.name': 'qwen_llm',
          'profile.llm_model.temperature': 0.5,
           'profile.llm_model.model_name': 'qwen2-72b-instruct',
           'action.tool': ['google_search_tool']
           })
    """
    yaml = YAML()
    # read an existing YAML file
    with open(config_path, 'r', encoding='utf-8') as file:
        config_data = yaml.load(file)

    # batch update the value of the target key
    for path, new_value in updates.items():
        keys = path.split('.')
        d = config_data
        try:
            for key in keys[:-1]:
                if key not in d:
                    raise KeyError(f"Key '{key}' not found in the configuration")
                d = d[key]
            # modify the value of the target key
            d[keys[-1]] = new_value
        except Exception as e:
            print(f"Skipping update for '{path}': {e}")

    # write the modified content back to the YAML file
    with open(config_path, 'w', encoding='utf-8') as file:
        yaml.dump(config_data, file)


def write_yaml_file(file_path, config_data) -> None:
    """Writes the dictionary to a YAML file with the specified file path.
    
    Args:
        file_path (str): The path to the YAML file.
        config_data (dict): The dictionary to write to the YAML file.
    """
    yaml = YAML()
    os.makedirs(os.path.dirname(file_path), exist_ok=True)

    # Write the dictionary to the YAML file
    with open(file_path, 'w', encoding='utf-8') as yaml_file:
        yaml.dump(config_data, yaml_file)


def delete_yaml_file(file_path: str) -> None:
    """Deletes the YAML file with the specified file path.

    Args:
        file_path (str): The path to the YAML file.
    """
    try:
        if os.path.exists(file_path):
            os.remove(file_path)
            print(f"YAML file '{file_path}' has been deleted.")
        else:
            print(f"File '{file_path}' does not exist.")
    except Exception as e:
        print(f"An error occurred while trying to delete the file: {e}")


def read_yaml_file(file_path) -> Dict[str, Any]:
    """Reads the YAML file and returns the data as a dictionary.

    Args:
        file_path (str): The path to the YAML file.

    Returns:
        dict: The data from the YAML file as a dictionary.
    """
    yaml = YAML()

    with open(file_path, 'r', encoding='utf-8') as file:
        data = yaml.load(file)

    return data



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/6/5 15:33
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: trace.py
import asyncio
import functools
import inspect

from functools import wraps

from agentuniverse.base.context.framework_context_manager import FrameworkContextManager
from agentuniverse.base.util.monitor.monitor import Monitor
from agentuniverse.llm.llm_output import LLMOutput


def trace_llm(func):
    """Annotation: @trace_llm

    Decorator to trace the LLM invocation, add llm input and output to the monitor.
    """

    @wraps(func)
    async def wrapper_async(*args, **kwargs):
        # get llm input from arguments
        llm_input = _get_llm_input(func, *args, **kwargs)

        source = func.__qualname__

        # check whether the tracing switch is enabled
        self = llm_input.pop('self', None)

        if self and hasattr(self, 'name'):
            name = self.name
            if name is not None:
                source = name

        # add invocation chain to the monitor module.
        Monitor.add_invocation_chain({'source': source, 'type': 'llm'})

        if self and hasattr(self, 'tracing'):
            if self.tracing is False:
                return await func(*args, **kwargs)

        # invoke function
        result = await func(*args, **kwargs)
        # not streaming
        if isinstance(result, LLMOutput):
            # add llm invocation info to monitor
            Monitor().trace_llm_invocation(source=func.__qualname__, llm_input=llm_input, llm_output=result.text)
            return result
        else:
            # streaming
            async def gen_iterator():
                llm_output = []
                async for chunk in result:
                    llm_output.append(chunk.text)
                    yield chunk
                # add llm invocation info to monitor
                Monitor().trace_llm_invocation(source=func.__qualname__, llm_input=llm_input,
                                               llm_output="".join(llm_output))

            return gen_iterator()

    @functools.wraps(func)
    def wrapper_sync(*args, **kwargs):

        # get llm input from arguments
        llm_input = _get_llm_input(func, *args, **kwargs)

        source = func.__qualname__

        # check whether the tracing switch is enabled
        self = llm_input.pop('self', None)
        if self and hasattr(self, 'name'):
            name = self.name
            if name is not None:
                source = name

        # add invocation chain to the monitor module.
        Monitor.add_invocation_chain({'source': source, 'type': 'llm'})

        if self and hasattr(self, 'tracing'):
            if self.tracing is False:
                return func(*args, **kwargs)

        # invoke function
        result = func(*args, **kwargs)
        # not streaming
        if isinstance(result, LLMOutput):
            # add llm invocation info to monitor
            Monitor().trace_llm_invocation(source=func.__qualname__, llm_input=llm_input, llm_output=result.text)

            # add llm token usage to monitor
            trace_llm_token_usage(self, llm_input, result.text)

            return result
        else:
            # streaming
            def gen_iterator():
                llm_output = []
                for chunk in result:
                    llm_output.append(chunk.text)
                    yield chunk

                output_str = "".join(llm_output)

                # add llm invocation info to monitor
                Monitor().trace_llm_invocation(source=func.__qualname__, llm_input=llm_input,
                                               llm_output=output_str)

                # add llm token usage to monitor
                trace_llm_token_usage(self, llm_input, output_str)

            return gen_iterator()

    if asyncio.iscoroutinefunction(func):
        # async function
        return wrapper_async
    else:
        # sync function
        return wrapper_sync


def _get_llm_input(func, *args, **kwargs) -> dict:
    """Get the llm input from arguments."""
    sig = inspect.signature(func)
    bound_args = sig.bind(*args, **kwargs)
    bound_args.apply_defaults()
    return {k: v for k, v in bound_args.arguments.items()}


def trace_agent(func):
    """Annotation: @trace_agent

    Decorator to trace the agent invocation, add agent input and output to the monitor.
    """

    @functools.wraps(func)
    async def wrapper_async(*args, **kwargs):
        # get agent input from arguments
        agent_input = _get_input(func, *args, **kwargs)
        # check whether the tracing switch is enabled
        source = func.__qualname__
        self = agent_input.pop('self', None)

        tracing = None
        if isinstance(self, object):
            agent_model = getattr(self, 'agent_model', None)
            if isinstance(agent_model, object):
                info = getattr(agent_model, 'info', None)
                profile = getattr(agent_model, 'profile', None)
                if isinstance(info, dict):
                    source = info.get('name', None)
                if isinstance(profile, dict):
                    tracing = profile.get('tracing', None)

        # add invocation chain to the monitor module.
        Monitor.add_invocation_chain({'source': source, 'type': 'agent'})

        if tracing is False:
            return await func(*args, **kwargs)

        # invoke function
        result = await func(*args, **kwargs)
        # add agent invocation info to monitor
        Monitor().trace_agent_invocation(source=source, agent_input=agent_input, agent_output=result)
        return result

    @functools.wraps(func)
    def wrapper_sync(*args, **kwargs):
        # get agent input from arguments
        agent_input = _get_input(func, *args, **kwargs)
        # check whether the tracing switch is enabled
        source = func.__qualname__
        self = agent_input.pop('self', None)

        tracing = None
        if isinstance(self, object):
            agent_model = getattr(self, 'agent_model', None)
            if isinstance(agent_model, object):
                info = getattr(agent_model, 'info', None)
                profile = getattr(agent_model, 'profile', None)
                if isinstance(info, dict):
                    source = info.get('name', None)
                if isinstance(profile, dict):
                    tracing = profile.get('tracing', None)

        # add invocation chain to the monitor module.
        Monitor.add_invocation_chain({'source': source, 'type': 'agent'})

        if tracing is False:
            return func(*args, **kwargs)

        # invoke function
        result = func(*args, **kwargs)
        # add agent invocation info to monitor
        Monitor().trace_agent_invocation(source=source, agent_input=agent_input, agent_output=result)
        return result

    if asyncio.iscoroutinefunction(func):
        # async function
        return wrapper_async
    else:
        # sync function
        return wrapper_sync


def trace_tool(func):
    """Annotation: @trace_tool

    Decorator to trace the tool invocation.
    """

    @functools.wraps(func)
    def wrapper_sync(*args, **kwargs):
        # get tool input from arguments
        tool_input = _get_input(func, *args, **kwargs)

        source = func.__qualname__
        self = tool_input.pop('self', None)

        if isinstance(self, object):
            name = getattr(self, 'name', None)
            if name is not None:
                source = name

        # add invocation chain to the monitor module.
        Monitor.add_invocation_chain({'source': source, 'type': 'tool'})

        # invoke function
        return func(*args, **kwargs)

    # sync function
    return wrapper_sync


def trace_knowledge(func):
    """Annotation: @trace_knowledge

    Decorator to trace the knowledge invocation.
    """

    @functools.wraps(func)
    def wrapper_sync(*args, **kwargs):
        # get knowledge input from arguments
        knowledge_input = _get_input(func, *args, **kwargs)

        source = func.__qualname__
        self = knowledge_input.pop('self', None)

        if isinstance(self, object):
            name = getattr(self, 'name', None)
            if name is not None:
                source = name

        # add invocation chain to the monitor module.
        Monitor.add_invocation_chain({'source': source, 'type': 'knowledge'})

        # invoke function
        return func(*args, **kwargs)

    # sync function
    return wrapper_sync


def _get_input(func, *args, **kwargs) -> dict:
    """Get the agent input from arguments."""
    sig = inspect.signature(func)
    bound_args = sig.bind(*args, **kwargs)
    bound_args.apply_defaults()
    return {k: v for k, v in bound_args.arguments.items()}


def _get_llm_token_usage(llm_obj: object, llm_input: dict, output_str: str) -> dict:
    """ Calculate the token usage of the given LLM object.
    Args:
        llm_obj(object): LLM object.
        llm_input(dict): Dictionary of LLM input.
        output_str(str): LLM output.

    Returns:
        dict: Dictionary of token usage including the completion_tokens, prompt_tokens, and total_tokens.
    """
    try:
        if llm_obj is None or llm_input is None:
            return {}
        messages = llm_input.get('kwargs', {}).pop('messages', None)

        input_str = ''
        if messages is not None and isinstance(messages, list):
            for m in messages:
                if isinstance(m, dict):
                    input_str += str(m.get('role', '')) + '\n'
                    input_str += str(m.get('content', '')) + '\n'

                elif isinstance(m, object):
                    if hasattr(m, 'role'):
                        role = m.role
                        if role is not None:
                            input_str += str(m.role) + '\n'
                    if hasattr(m, 'content'):
                        content = m.content
                        if content is not None:
                            input_str += str(m.content) + '\n'

        if input_str == '' or output_str == '':
            return {}

        usage = {}
        # the number of input and output tokens is calculated by the llm `get_num_tokens` method.
        if hasattr(llm_obj, 'get_num_tokens'):
            completion_tokens = llm_obj.get_num_tokens(output_str)
            prompt_tokens = llm_obj.get_num_tokens(input_str)
            total_tokens = completion_tokens + prompt_tokens
            usage = {'completion_tokens': completion_tokens, 'prompt_tokens': prompt_tokens,
                     'total_tokens': total_tokens}
        return usage
    except Exception as e:
        return {}


def trace_llm_token_usage(llm_obj: object, llm_input: dict, output_str: str) -> None:
    """ Trace the token usage of the given LLM object.
    Args:
        llm_obj(object): LLM object.
        llm_input(dict): Dictionary of LLM input.
        output_str(str): LLM output.
    """
    trace_id = FrameworkContextManager().get_context('trace_id')
    # trace token usage for a complete request chain based on trace id
    if trace_id:
        token_usage: dict = _get_llm_token_usage(llm_obj, llm_input, output_str)
        if token_usage:
            Monitor.add_token_usage(token_usage)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/10/25 11:20
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: rag_agent.py
from agentuniverse.agent.template.rag_agent_template import RagAgentTemplate


class RagAgent(RagAgentTemplate):
    """Rag Agent class."""



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/4/19 15:59
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: rpc_server.py

import json

from agentuniverse.agent_serve.service_instance import ServiceInstance
from ..request_task import RequestTask
from ..web_util import service_run_queue


def service_run(saved: bool, params: str, service_id: str):
    """Synchronous invocation of an agent service. Used in rpc implementation.

    Request Args:
        service_id(`str`): The id of the agent service.
        params(`str`): A Json String contains agent params passed to service.
        saved(`bool`): Save the request and result into database.

    Return:
        Returns a dict containing two keys: success and result.
        success: This key holds a boolean value indicating the task was
            successfully or not.
        result: This key points to a nested dictionary that includes the
            result of the task.
    """
    if params and params.strip():
        params = json.loads(params)
    else:
        params = {}
    request_task = RequestTask(ServiceInstance(service_id).run, saved,
                               **params)
    result = request_task.run()
    return {
        "success": True,
        "result": result,
        "message": None,
        "request_id": request_task.request_id
    }


def service_run_async(saved: bool, params: str, service_id: str):
    """Async invocation of an agent service, return the request id used to
    get result later. Used in rpc implementation.

    Request Args:
        service_id(`str`): The id of the agent service.
        params(`str`): A Json String contains agent params passed to service.
        saved(`bool`): Save the request and result into database.

    Return:
        Returns a dict containing two keys: success and request_id.
        success: This key holds a boolean value indicating the task was
            successfully or not.
        request_id: Stand for a single request taski, can be used in
            service_run_result api to get the result of async task.
    """
    if params and params.strip():
        params = json.loads(params)
    else:
        params = {}
    params['service_id'] = service_id
    task = RequestTask(service_run_queue, saved, **params)
    task.async_run()
    return {
        "success": True,
        "result": None,
        "message": None,
        "request_id": task.request_id
    }


def service_run_result(request_id: str):
    """Get the async service result.

    Request Args:
        request_id(`str`): Request id returned by async run api.

    Return:
        Returns a dict containing two keys: success and result if request_id
        exists in database.
        success: This key holds a boolean value indicating the task was
            successfully or not.
        result: This key points to a nested dictionary that includes the
            result of the task.
    """
    data = RequestTask.query_request_state(request_id)
    if data is None:
        return {
            "success": False,
            "message": f"request {request_id} not found"
        }
    return {
        "success": True,
        "result": json.dumps(data, ensure_ascii=False),
        "message": None,
        "request_id": request_id
    }


# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/14 15:35
# @Author  : jerry.zzw 
# @Email   : jerry.zzw@antgroup.com
# @FileName: component_base.py
from typing import Optional

from pydantic import BaseModel, ConfigDict

from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.config.application_configer.application_config_manager import ApplicationConfigManager
from agentuniverse.base.config.component_configer.component_configer import ComponentConfiger


class ComponentBase(BaseModel):
    """The ComponentBase class, which is used to define the base class of the component."""

    component_type: ComponentEnum
    # component yaml path
    component_config_path: Optional[str] = None

    # Used to indicate whether this object is the default object for the corresponding component.
    default_symbol: bool = False
    # pydantic protected_namespaces config
    model_config = ConfigDict(protected_namespaces=())
    def get_instance_code(self) -> str:
        """Return the full name of the component."""
        appname = ApplicationConfigManager().app_configer.base_info_appname
        return f'{appname}.{self.component_type.value.lower()}.{self.name}'

    def initialize_by_component_configer(self, component_configer: ComponentConfiger) -> 'ComponentBase':
        """Initialize the component by the ComponentConfiger object.

        Args:
            component_configer(ComponentConfiger): the ComponentConfiger object
        Returns:
            ComponentBase: the component object
        """
        if hasattr(component_configer, "default_symbol"):
            self.default_symbol = component_configer.default_symbol
        self._initialize_by_component_configer(component_configer)

        return self

    def _initialize_by_component_configer(self, component_configer: ComponentConfiger) -> 'ComponentBase':
        pass

    def is_default_object(self):
        return self.default_symbol



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/8/23 15:52
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: workflow_manager.py
from agentuniverse.base.annotation.singleton import singleton
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.component.component_manager_base import ComponentManagerBase


@singleton
class WorkflowManager(ComponentManagerBase):
    """The WorkflowManager class, which is used to manage workflow."""

    def __init__(self):
        super().__init__(ComponentEnum.WORKFLOW)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: agentuniverse_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x61gentuniverse_service.proto\x12\ragentuniverse\"H\n\x13\x41gentServiceRequest\x12\x12\n\nservice_id\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x01(\t\x12\r\n\x05saved\x18\x03 \x01(\x08\"\\\n\x14\x41gentServiceResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0f\n\x07success\x18\x02 \x01(\x08\x12\x12\n\nrequest_id\x18\x03 \x01(\t\x12\x0e\n\x06result\x18\x04 \x01(\t\"(\n\x12\x41gentResultRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t2\xaa\x02\n\x14\x41gentUniverseService\x12V\n\x0bservice_run\x12\".agentuniverse.AgentServiceRequest\x1a#.agentuniverse.AgentServiceResponse\x12\\\n\x11service_run_async\x12\".agentuniverse.AgentServiceRequest\x1a#.agentuniverse.AgentServiceResponse\x12\\\n\x12service_run_result\x12!.agentuniverse.AgentResultRequest\x1a#.agentuniverse.AgentServiceResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'agentuniverse_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_AGENTSERVICEREQUEST']._serialized_start=46
  _globals['_AGENTSERVICEREQUEST']._serialized_end=118
  _globals['_AGENTSERVICERESPONSE']._serialized_start=120
  _globals['_AGENTSERVICERESPONSE']._serialized_end=212
  _globals['_AGENTRESULTREQUEST']._serialized_start=214
  _globals['_AGENTRESULTREQUEST']._serialized_end=254
  _globals['_AGENTUNIVERSESERVICE']._serialized_start=257
  _globals['_AGENTUNIVERSESERVICE']._serialized_end=555
# @@protoc_insertion_point(module_scope)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/28 11:33
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: message.py
from typing import Optional, List, Union, Dict

from langchain_core.messages import HumanMessage
from langchain_core.prompts import SystemMessagePromptTemplate, HumanMessagePromptTemplate, AIMessagePromptTemplate
from langchain_core.prompts.chat import BaseStringMessagePromptTemplate
from pydantic import BaseModel

from agentuniverse.agent.memory.enum import ChatMessageEnum


class Message(BaseModel):
    """The basic class for memory message.

    Attributes:
        id (int): The id of the message.
        type (Optional[str]): The type of the message.
        content (Optional[str]): The content of the message.
        source (Optional[str]): The source of the message.
        metadata (Optional[dict]): The metadata of the message.
    """
    id: Optional[str] = None
    type: Optional[str] = None
    content: Optional[Union[str, List[Union[str, Dict]]]] = None
    source: Optional[str] = None
    metadata: Optional[dict] = None

    def as_langchain(self):
        """Convert the agentUniverse(aU) message class to the langchain message class."""
        if self.type == ChatMessageEnum.SYSTEM.value:
            return SystemMessagePromptTemplate.from_template(self.content)
        elif self.type == ChatMessageEnum.HUMAN.value:
            if isinstance(self.content, str):
                return HumanMessagePromptTemplate.from_template(self.content)
            elif isinstance(self.content, list):
                return HumanMessage(content=self.content)
        elif self.type == ChatMessageEnum.AI.value:
            return AIMessagePromptTemplate.from_template(self.content)
        else:
            return BaseStringMessagePromptTemplate.from_template(self.content)

    @staticmethod
    def as_langchain_list(message_list: List['Message']):
        """Convert agentUniverse(aU) message list to langchain message list """
        langchain_message_list = []
        if message_list is None:
            return langchain_message_list
        for message in message_list:
            langchain_message_list.append(message.as_langchain())
        return langchain_message_list

    def to_dict(self) -> dict:
        """Convert the agentUniverse(aU) message class to the dict."""
        return {"type": self.type, "content": self.content, "metadata": self.metadata, "source": self.source}

    @staticmethod
    def from_dict(message_dict: dict) -> 'Message':
        """Convert the dict to agentUniverse(aU) message class.

        Args:
            message_dict (dict): The dict of the message.
        Returns:
            Message: The agentUniverse(aU) message class.
        """
        message = Message()
        attributes = ['id', 'content', 'type', 'source', 'metadata']
        for attr in attributes:
            if attr in message_dict:
                setattr(message, attr, message_dict[attr])
        return message



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/28 10:18
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: web_booster.py

import sys
import threading

from .post_fork_queue import POST_FORK_QUEUE

ACTIVATE_OPTIONS = {
    "gunicorn": False,
    "grpc": False
}


def start_web_server(**kwargs):
    """
    Start func of web server, include http server and grpc server. Use Flask
    as default http server.
    The gRPC server is not enabled by default; it needs
    to be configured in the configuration file.
    Accept input arguments to overwrite default config.
    """
    # Start grpc server.
    if ACTIVATE_OPTIONS["grpc"]:
        from .rpc.grpc.grpc_server_booster import start_grpc_server
        grpc_thread = threading.Thread(
            target=start_grpc_server
        )
        grpc_thread.start()

    # Start http server.
    if ACTIVATE_OPTIONS["gunicorn"]:
        from .gunicorn_server import GunicornApplication
        GunicornApplication().update_config(kwargs)
        GunicornApplication().run()
    else:
        from .flask_server import app
        if 'bind' in kwargs:
            host, port = kwargs['bind'].split(':')
            port = int(port)
        else:
            port = 8888
            host = '0.0.0.0'
        for _func, args, kwargs in POST_FORK_QUEUE:
            _func(*args, **kwargs)
        app.run(port=port, host=host, debug=False)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/13 14:34
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: enum.py
import enum
from enum import Enum


@enum.unique
class ToolTypeEnum(Enum):
    API = 'api'
    FUNC = 'func'



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/3/12 15:48
# @Author  : fanen.lhy
# @Email   : fanen.lhy@antgroup.com
# @FileName: logging_util.py

import os
import sys
from typing import Optional
from pathlib import Path

import loguru

from .general_logger import GeneralLogger, LOG_LEVEL
from .logging_config import LoggingConfig, init_log_config
from ..system_util import get_project_root_path

_module_logger_dict = {}
STANDARD_LOG_SUFFIX = 'all'
ERROR_LOG_SUFFIX = 'error'
LOG_FILE_PREFIX = "au"
LOG_SUB_DIR = 'logs'
LOGGER = GeneralLogger(STANDARD_LOG_SUFFIX, "", "", "", "", add_handler=False)


def _get_log_file_path(log_suffix: str) -> str:
    """Get full log file path by contacting log save path and log file name.
    If log path is not specified, create a subdir to save logs under project
    root dir.

    Args:
        log_suffix (`str`):
            Suffix of the log file name.

    Returns:
        Full log file path string.
    """
    if LoggingConfig.log_path:
        project_log_dir = Path(LoggingConfig.log_path)
    else:
        project_path = get_project_root_path()
        project_log_dir = project_path / LOG_SUB_DIR
    project_log_filename = f'{LOG_FILE_PREFIX}_{log_suffix}.log'
    project_log_path = project_log_dir / project_log_filename
    return str(project_log_path)


def _add_standard_logger():
    """Add a standard loguru handler."""
    LOGGER.update_properties(
        log_path=_get_log_file_path(STANDARD_LOG_SUFFIX),
        log_format=LoggingConfig.log_format,
        log_rotation=LoggingConfig.log_rotation,
        log_retention=LoggingConfig.log_retention)
    loguru.logger.add(
        sink=_get_log_file_path(STANDARD_LOG_SUFFIX),
        level=LoggingConfig.log_level,
        format=LoggingConfig.log_format,
        rotation=LoggingConfig.log_rotation,
        retention=LoggingConfig.log_retention,
        compression='zip',
        encoding="utf-8",
        enqueue=True
    )


def _add_std_out_handler():
    """Add a loguru handler send logs to std out."""
    loguru.logger.add(
        sink=sys.stdout,
        level=LoggingConfig.log_level,
        format=LoggingConfig.log_format,
        enqueue=True
    )


def _add_error_log_handler():
    """Add a loguru handler to record all error level message."""
    loguru.logger.add(
        sink=_get_log_file_path(ERROR_LOG_SUFFIX),
        level="ERROR",
        format=LoggingConfig.log_format,
        rotation=LoggingConfig.log_rotation,
        retention=LoggingConfig.log_retention,
        compression='zip',
        encoding="utf-8",
        enqueue=True,
    )


def _add_sls_log_handler():
    """Add a handler to record all """
    from agentuniverse_extension.logger.sls_sink import SlsSink, SlsSender
    sls_sender = SlsSender(LoggingConfig.sls_project,
                           LoggingConfig.sls_log_store,
                           LoggingConfig.sls_endpoint,
                           LoggingConfig.access_key_id,
                           LoggingConfig.access_key_secret,
                           LoggingConfig.sls_log_queue_max_size,
                           LoggingConfig.sls_log_send_interval)
    sls_sender.start_batch_send_thread()
    loguru.logger.add(
        sink=SlsSink(sls_sender),
        format=LoggingConfig.log_format,
        level=LoggingConfig.log_level,
        enqueue=True
    )


def get_module_logger(module_name: str,
                      log_level: Optional[LOG_LEVEL] = None) -> GeneralLogger:
    """Get a module dedicated logger.

    Args:
        module_name (`str`):
            Name of the module, also used in log file name.
        log_level (`LOG_LEVEL`, defaults to `None`):
            Log level, values should be in LOG_LEVEL. If log_level is none,
            use log level in logging config as default value.

    Returns:
        Return a logger used to record specify module log.
    """
    if module_name in _module_logger_dict:
        return _module_logger_dict.get(module_name)
    if not log_level:
        log_level = LoggingConfig.log_level
    new_logger = GeneralLogger(module_name,
                               _get_log_file_path(module_name),
                               LoggingConfig.log_format,
                               LoggingConfig.log_rotation,
                               LoggingConfig.log_retention,
                               log_level)
    _module_logger_dict[module_name] = new_logger
    return new_logger


def add_sink(sink, log_level: Optional[LOG_LEVEL] = None) -> bool:
    """Validate the given sink and add it to the loguru logger if valid.

    Args:
        sink :
            A filepath (str), file object, object with `write` method,
            or callable function with only one param named message.
        log_level (`LOG_LEVEL`, defaults to `None`):
            Log level, values should be in LOG_LEVEL. If log_level is none,
            use log level in logging config as default value.
    Return:
        True if the sink was successfully added, False otherwise.
    """
    valid_sink = False

    if isinstance(sink, str):
        # If the sink is a string, assume it's a file path
        valid_sink = True

    elif hasattr(sink, 'write') and callable(getattr(sink, 'write')):
        # An object with a write method (like file objects or class instances)
        valid_sink = True

    elif callable(sink):
        # A callable object, such as a function or a lambda
        valid_sink = True

    if valid_sink:
        try:
            loguru.logger.add(
                sink=sink,
                format=LoggingConfig.log_format,
                level=log_level or LoggingConfig.log_level,
                enqueue=True
            )
            return True
        except Exception as e:
            print(f"Failed to add log sink due to error: {e}")

    print(
        f"The provided sink type {type(sink)} is not supported "
        f"or caused an error.")
    return False


def init_loggers(config_path: Optional[str] = None):
    """Parse config and initialize all loggers and handlers.

    Args:
        config_path (`str`):
            Path of the log config file.
    """
    init_log_config(config_path)
    loguru.logger.remove()
    _add_std_out_handler()
    _add_error_log_handler()
    if LoggingConfig.log_extend_module_switch["sls_log"]:
        _add_sls_log_handler()
    _add_standard_logger()



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/5/14 14:41
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: chat_prompt.py
import base64
from typing import List
import re
from urllib.parse import urlparse

from langchain_core.prompts import ChatPromptTemplate

from agentuniverse.agent.memory.enum import ChatMessageEnum
from agentuniverse.agent.memory.message import Message
from agentuniverse.base.util.prompt_util import generate_chat_template
from agentuniverse.prompt.prompt import Prompt
from agentuniverse.prompt.prompt_model import AgentPromptModel

image_extensions = (
    ".png",
    ".jpg",
    ".jpeg",
    ".gif",
    ".webp",
)


class ChatPrompt(Prompt):
    messages: List[Message] = []

    def as_langchain(self) -> ChatPromptTemplate:
        return ChatPromptTemplate.from_messages(Message.as_langchain_list(self.messages))

    def build_prompt(self, agent_prompt_model: AgentPromptModel, prompt_assemble_order: list[str]) -> 'ChatPrompt':
        """Build the prompt class.

        Args:
            agent_prompt_model (AgentPromptModel): The user agent prompt model.
            prompt_assemble_order (list[str]): The prompt assemble ordered list.

        Returns:
            ChatPrompt: The chat prompt object.
        """
        self.messages = generate_chat_template(agent_prompt_model, prompt_assemble_order)
        self.input_variables = self.extract_placeholders()
        return self

    def extract_placeholders(self) -> List[str]:
        """Extract the placeholders from the messages.

        Returns:
            List[str]: The placeholders list.
        """
        result = []
        placeholder_pattern = re.compile(r'\{(.*?)}')
        for message in self.messages:
            matches = placeholder_pattern.findall(message.content)
            result.extend(matches)
        return result

    def generate_image_prompt(self, image_urls: list[str]) -> None:
        """ Generate the prompt with image urls.

        Args:
            image_urls (list[str]): The image urls.
        """
        if image_urls:
            for image_url in image_urls:
                parsed_url = urlparse(image_url)
                # Check if the URL is a valid HTTP or HTTPS URL.
                if parsed_url.scheme in ["http", "https"]:
                    content = [{"type": "image_url", "image_url": {"url": image_url}}]
                    self.messages.append(Message(type=ChatMessageEnum.HUMAN.value, content=content))
                # Check if the URL is a local file.
                elif parsed_url.scheme == "file" or not parsed_url.scheme:
                    if parsed_url.path.lower().endswith(image_extensions):
                        with open(parsed_url.path, "rb") as image_file:
                            base64_image = base64.b64encode(image_file.read()).decode(
                                "utf-8",
                            )
                            extension = parsed_url.path.lower().split(".")[-1]
                            mime_type = f"image/{extension}"
                            content = [
                                {"type": "image_url", "image_url": {"url": f"data:{mime_type};base64,{base64_image}"}}]
                            self.messages.append(Message(type=ChatMessageEnum.HUMAN.value, content=content))



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/10/17 11:55
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: work_pattern_manager.py
from agentuniverse.agent.work_pattern.work_pattern import WorkPattern
from agentuniverse.base.annotation.singleton import singleton
from agentuniverse.base.component.component_enum import ComponentEnum
from agentuniverse.base.component.component_manager_base import ComponentManagerBase


@singleton
class WorkPatternManager(ComponentManagerBase[WorkPattern]):
    """A singleton manager class of the WorkPattern."""

    def __init__(self):
        super().__init__(ComponentEnum.WORK_PATTERN)



# !/usr/bin/env python3
# -*- coding:utf-8 -*-

# @Time    : 2024/7/26 10:42
# @Author  : wangchongshi
# @Email   : wangchongshi.wcs@antgroup.com
# @FileName: prompt_dto.py
from typing import Optional

from pydantic import BaseModel, Field


class PromptDTO(BaseModel):
    introduction: Optional[str] = Field(description="prompt introduction", default="")
    target: Optional[str] = Field(description="prompt target", default="")
    instruction: Optional[str] = Field(description="prompt instruction", default="")
